#ifndef APPLICATION_H
#define APPLICATION_H


class Benchmarker;
#include "ExampleApplication.h"
#include "KeyListener.h"

namespace Ogre
{
	class Entity;
	class Pass;
	class SceneNode;
}
#include "OgreFrameListener.h"
#include "OgreVector3.h"
#include <vector>


class Application: public ExampleApplication, KeyListener, Ogre::FrameListener
{
private:
	struct MeshName
	{
		MeshName(std::string name, std::string description, const Ogre::Vector3& lookAtCentre, float cameraDistance):
			name(name),
			description(description),
			lookAtCentre(lookAtCentre),
			cameraDistance(cameraDistance){}
		std::string name;
		std::string description;
		Ogre::Vector3 lookAtCentre;
		float cameraDistance;
	};
	struct MaterialName
	{
		MaterialName(std::string name, std::string description): name(name), description(description){}
		std::string name;
		std::string description;
	};
public:
	Application();
	virtual ~Application();

	virtual void createScene();
	virtual void createFrameListener();

	virtual void keyPressed(const OIS::KeyEvent& e);
	virtual bool frameStarted(const Ogre::FrameEvent& evt);

	size_t getNumMaterials() const;
	std::string getCurrentMaterialName() const;
	void switchMaterial(bool nextMaterial);
	void switchToMaterial(size_t materialIndex);
	void addToNumObjects(int amount);
	void setNumObjects(int numObjects);
	size_t getNumMeshes() const;
	void switchMesh(bool nextMesh);
	void switchToMesh(size_t meshIndex);
	void setWallFrequences(const Ogre::Vector3& wallFrequencies);
	void setOverlayTexts(const std::string& text0, const std::string& text1) const;
private:
	void pushMaterial(const std::string& materialName, const std::string& description);
	void destroyObjects();
	void createObjects();
	void rotateObjects(Ogre::Degree amount);
	bool currentMeshHasSwappableMaterial() const;
	bool meshHasSwappableMaterial(size_t meshIndex) const;
	void setEntitiesMaterial();
	Ogre::Pass* getPass() const;
	void updateKeysDown(float time);
	void updateUniformParameters();
	void updateCameraAnimation(float time);
	bool hasParameter(const std::string& parameterName) const;
	void updateOverlayText() const;
	void updateFullInformation(std::stringstream& text0, std::stringstream& text1) const;
	void updateSmallInformation(std::stringstream& text0) const;
	static std::string toUpperCase(std::string text);

	int mNumObjects;
	std::vector<MeshName> mMeshes;
	size_t mMeshesIndex;
	std::vector<Ogre::Entity*> mEntities;
	std::vector<Ogre::SceneNode*> mSceneNodes;

	std::vector<MaterialName> mMaterials;
	size_t mMaterialIndex;
	Ogre::Vector3 mDefaultWallFrequencies, mWallFrequencies;
	float mDefaultLightThreshold, mLightThreshold;
	float mDefaultAlphaPlaneDistance, mAlphaPlaneDistance;
	float mDefaultWallDisplacement, mWallDisplacement;
	const std::string mWallFrequenciesString, mLightThresholdString;
	std::string mAlphaPlaneDistanceString, mWallDisplacementString;

	enum CameraMode
	{
		CM_AUTOMATIC,
		CM_FREE
	};
	CameraMode mCameraMode;
	float mCameraAnimationPhase;

	Benchmarker* mBenchmarker;

	enum InfoState
	{
		IS_SMALL,
		IS_FULL,
		IS_NONE
	};
	InfoState mInfoState;
};


#endif //APPLICATION_H