#include "Logger.h"

#include "OgreLogManager.h"
#include "OgreMovableObject.h"
#include "OgreSceneNode.h"


Ogre::Log* const Logger::getLog()
{
	static Ogre::Log* log;
	if (!log)
	{
		log = Ogre::LogManager::getSingleton().createLog("Application.log");
	}
	return log;
}


void Logger::log(std::string text)
{
	getLog()->logMessage(text);
}


void Logger::log(Ogre::SceneNode* node)
{
	std::stringstream ss;

    ss << std::endl << "Node Hierarchy:" << std::endl;
    dumpNodes(ss, node, 0);
    log(ss.str());
}


//original code for this function by Kojack, taken from
//http://www.ogre3d.org/wiki/index.php/DumpingNodeTree
void Logger::dumpNodes(std::stringstream &ss, Ogre::Node* n, int level)
{
	for(int i = 0; i < level; i++)
	{
		ss << " ";
	}
	ss << "SceneNode: " << n->getName() << std::endl;

	Ogre::SceneNode::ObjectIterator object_it = ((Ogre::SceneNode*)n)->getAttachedObjectIterator();
	Ogre::Node::ChildNodeIterator node_it = n->getChildIterator();

	Ogre::MovableObject *m;
	while(object_it.hasMoreElements())
	{
		for(int i = 0; i < level + 2; i++)
		{
		ss << " ";
		}
		m = object_it.getNext();
		ss << m->getMovableType() << ": " << m->getName() << std::endl;
	}
	while(node_it.hasMoreElements())
	{
		dumpNodes(ss, node_it.getNext(), level + 2);
	}
}