#ifndef LOGGER_H
#define LOGGER_H


namespace Ogre
{
	class Log;
	class SceneNode;
	class Node;
}
#include <sstream>
#include <string>


/**
* A static class for quickly writing data to a log-file for debugging.
* This class accesses the log-file for every writing, so using this a lot every frame
* will heavily hurt the framerate.
*/
class Logger
{
public:

	static Ogre::Log* const getLog();

	static void log(Ogre::SceneNode* node);

	static void log(std::string text);

	template <typename T>
	static void log(const T info);

	template <typename S, typename T>
	static void log(const S part1, const T part2);

	template <typename S, typename T, typename U>
	static void log(const S part1, const T part2, const U part3);

	template <typename S, typename T, typename U, typename V>
	static void log(const S part1, const T part2, const U part3, const V part4);

protected:

	static void dumpNodes(std::stringstream &ss, Ogre::Node* n, int level);
};


template <typename T>
static void Logger::log(const T info)
{
	std::stringstream output;
	output << info;
	log(output.str());
}

template <typename S, typename T>
static void Logger::log(const S part1, const T part2)
{
	std::stringstream output;
	output << part1 << "  " << part2;
	log(output.str());
}

template <typename S, typename T, typename U>
static void Logger::log(const S part1, const T part2, const U part3)
{
	std::stringstream output;
	output << part1 << "  " << part2 << "  " << part3;
	log(output.str());
}

template <typename S, typename T, typename U, typename V>
static void Logger::log(const S part1, const T part2, const U part3, const V part4)
{
	std::stringstream output;
	output << part1 << "  " << part2 << "  " << part3 << "  " << part4;
	log(output.str());
}


#endif //LOGGER_H